/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "about.h" 
#include "variables.h"

#include <gtkmm/image.h>

// About OpenLDev message dialog
about_dialog::about_dialog()
{
  Glib::ustring str = "OpenLDev " + Glib::ustring(VERSION);
  
  initialize_objects(str);
  dialog->set_default_icon_from_file("/usr/share/pixmaps/openldev/openldev24.png");
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  
  // Load OpenLDev logo & resize it
  Glib::RefPtr<Gdk::Pixbuf> pixbuf = Gdk::Pixbuf::create_from_file("/usr/share/pixmaps/openldev/openldev48.png");
  pixbuf = pixbuf->scale_simple(96, 96, Gdk::INTERP_BILINEAR);
  Gtk::Image *logo = new Gtk::Image(pixbuf);
  
  // Setup text
  copyright_label->set_justify(Gtk::JUSTIFY_CENTER);
  openldev_label->set_justify(Gtk::JUSTIFY_CENTER);
  Pango::FontDescription fd("Sans Bold 18");  
  openldev_label->modify_font(fd);
  ok->set_flags(Gtk::CAN_DEFAULT);
  
  // Pack text & textview
  vbox->pack_start(*logo, Gtk::PACK_SHRINK, 3);
  vbox->pack_start(*openldev_label, Gtk::PACK_SHRINK, 3);
  vbox->pack_start(*copyright_label, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*hseparator, Gtk::PACK_SHRINK, 10);
  
  // Create & view about box
  dialog->get_vbox()->pack_start(*vbox);
  dialog->set_title("About OpenLDev");
  dialog->set_border_width(5);
  dialog->set_resizable(true);
  dialog->set_has_separator(false);
  dialog->add_action_widget(*ok, -5);
  dialog->set_default(*ok);

  dialog->show_all_children();
  dialog->show();
}

// Delete all widgets to clean up space
about_dialog::~about_dialog()
{
  delete openldev_label;
  delete copyright_label;
  delete vbox;
  delete ok;
  delete hseparator;
}

// Create dialog objects
void about_dialog::initialize_objects(Glib::ustring str)
{
  dialog = this;
  openldev_label = new Gtk::Label(str);
  copyright_label = new Gtk::Label(COPYRIGHT);
  hseparator = new Gtk::HSeparator();
  vbox = new Gtk::VBox(false, 0);
  ok = new Gtk::Button(Gtk::StockID("gtk-close"));
}
