/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "build.h"
#include "utils.h"

bool continue_build;
bool run_command(Glib::ustring command, bool catch_output, 
                 Gtk::TreeView *build_box, environment_settings env);

// Run autogen.sh
void openldev::on_autogen()
{  
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    chdir (proj.get_location().c_str());
    Glib::ustring command = "./autogen.sh 2>&1";
    run_command(command, true, build_box, env);
  }
}

// Compile current file
void openldev::on_compile() 
{
  if (noteman->get_current_filename().empty()) 
  {
    error("Please open or create\na file first!");
  } 
  else 
  {
    Glib::ustring command = "g";
    Glib::ustring location = Glib::ustring(proj.get_location()) + "src/";
    chdir (proj.get_location().c_str());
    
    // Patch together the 'compile' command
    if (proj.get_language() == '+')
      command += "++ -Wall -g -c \"";
    else
      command += "cc -Wall -g -c \"";
    command += noteman->get_current_filename() + "\" -o \"";
    
    // Add filename based (Need to check the length to cut off filename)
    if (noteman->get_current_filename().find(".cc",0) != Glib::ustring::npos)
      command += noteman->get_current_filename().substr(0, noteman->get_current_filename().length()-3);
    else if (noteman->get_current_filename().find(".cpp",0) != Glib::ustring::npos)
      command += noteman->get_current_filename().substr(0, noteman->get_current_filename().length()-4);
    else
      command += noteman->get_current_filename().substr(0, noteman->get_current_filename().length()-2);
    command += "\" 2>&1";
    
    run_command(command, true, build_box, env);
  }
}

// Build project
void openldev::on_build() 
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    noteman->save_all_files();
    chdir (proj.get_location().c_str());
    Glib::ustring command = "make 2>&1";
    if (!run_command(command, true, build_box, env))
    {
      on_autogen();
      run_command(command, true, build_box, env);
    }
  }
}

// Execute current file (TODO: Check if project is built)
void openldev::on_execute() 
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    Glib::ustring dir = Glib::ustring(proj.get_location()) + "src/";
    chdir (dir.c_str());
    Glib::ustring command = Glib::ustring(env.get_terminal()) + " -e " 
      + Glib::ustring(proj.get_location()) + "src/" + Glib::ustring(proj.get_name());
    run_command(command, false, build_box, env);
  }
}

// Run ./configure on the project
void openldev::on_configure() 
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    chdir (proj.get_location().c_str());
    Glib::ustring command = "./configure 2>&1";
    run_command(command, true, build_box, env);
  }
}

// Clean the project
void openldev::on_clean()
{
  if (proj.get_name().empty()) 
  {
    error("Please open or create\na project first!");
  } 
  else 
  {
    chdir (proj.get_location().c_str());
    Glib::ustring command = "make distclean 2>&1";
    run_command(command, true, build_box, env);
  }
}

void openldev::on_stop() 
{
  continue_build = false;
}

// Run the specified command with file pipes
bool run_command(Glib::ustring command, bool catch_output, 
                 Gtk::TreeView *build_box, environment_settings env)
{
  char buff[32];
  FILE *in;
  bool successful = true, autogenerate = false;
  unsigned int location;
  Glib::ustring text = "\0", line;
  BuildColumns build_columns;
  Glib::RefPtr<Gtk::ListStore> build_model;
  Gtk::CellRendererText *renderer;
  Gtk::TreeModel::Row row;
  Gtk::TreeViewColumn *color_column;
  continue_build = true;
  
  // Clean up build_box & get it ready to accept output
  build_box->remove_all_columns();
  build_model = Gtk::ListStore::create(build_columns);
  build_box->set_model(build_model);
  
  // Append the columns & create the cell renderer
  build_box->append_column("Compiler Output", build_columns.col_text);
  color_column = build_box->get_column(0);
  Gtk::CellRenderer *cell_renderer = color_column->get_first_cell_renderer();
  renderer = dynamic_cast<Gtk::CellRendererText*>(cell_renderer);
  color_column->add_attribute(renderer->property_foreground(), build_columns.col_color);
  
  // Add the command to the box
  row = *(build_model->append());
  row[build_columns.col_text] = command;
  row[build_columns.col_color] = env.get_warning_color();
  build_box->get_selection()->select(build_box->get_model()->get_iter("0"));
    
  // Open pipe & loop until the program is complete
  in = popen(command.c_str(), "r");
  
  if (catch_output)
  {
    while (fgets(buff, sizeof(buff), in) && continue_build) 
    {
      text += buff;
      // Find the endl's & split them to be added to build_box
      location = text.find("\n",0);
      while (location != Glib::ustring::npos) 
      {
        // Check to see if autogen.sh must be run before build
        if (text.find("No targets specified",0) != string::npos)
          autogenerate = true;
        
        // Append line & erase it from the buffer
        line = text.substr(0,location);
        row = *(build_model->append());
        row[build_columns.col_text] = line;
        text.erase(0,location+1);
        location = text.find("\n",0);

        // Apply Error or Warning colors
        if (line.find("error: ") != Glib::ustring::npos) 
        {
          row[build_columns.col_color] = env.get_error_color();
          successful = false;
        } 
        else if (line.find(": ") != Glib::ustring::npos)
          row[build_columns.col_color] = env.get_warning_color();
        else
          row[build_columns.col_color] = "#000000";
      
        // Update the build_box when there is output
        while(Gtk::Main::instance()->events_pending()) 
        {
          Gtk::Main::instance()->iteration();
          build_box->scroll_to_row(build_model->get_path(row));
        }
      }
    }
    
    // Close the stream, & print whether command was successful or not
    pclose(in);
    row = *(build_model->append());
    if (successful)
      row[build_columns.col_text] = "Completed ... Successful";
    else
      row[build_columns.col_text] = "Completed ... Failed";
    row[build_columns.col_color] = "#000000";
    build_box->scroll_to_row(build_model->get_path(row));
  }
  
  // Return false if the user must run autogen.sh before building & then do it
  if (autogenerate)
  {
    row = *(build_model->append());
    row[build_columns.col_text] = "You must run 'Auto Generate' first!";
    row[build_columns.col_color] = env.get_warning_color();
    return false;
  }
  return true;
}
