/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "environment.h"
#include "utils.h"
#include <gtkmm/adjustment.h>
#include <gtkmm/fontselection.h>
#include <gtkmm/messagedialog.h>

// Create environment properties dialog
environment_dialog::environment_dialog (environment_settings &env) 
{
  fontname = Glib::ustring(env.get_font_face()) + " " + Glib::ustring(env.get_font_size());
  
  // Create objects for dialog
  dialog = this;
  initialize_objects();
  
  dialog->set_default_icon_from_file("/usr/share/pixmaps/openldev/openldev24.png");
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  ok->set_flags(Gtk::CAN_DEFAULT);
  
  label_tabstops->set_alignment(0,0.5);
  label_filemanagement->set_alignment(0,0.5);
  label_terminal->set_alignment(0,0.5);
  label_tabwidth->set_padding(10,0);
  label_autosave->set_padding(10,0);
  label_minutes->set_padding(10,0);
  label_use_terminal->set_padding(10,0);
  tab_size->set_digits(0);
  tab_size->set_value(env.get_tabsize());
  asave_minutes->set_digits(0);
  asave_minutes->set_value(env.get_autosave());
  replace_tabs->set_active(env.get_replace_tabs());
  load_on_launch->set_active(env.get_autoload());
  terminal->set_text(env.get_terminal());
  hbox->pack_start(*label_tabwidth, Gtk::PACK_SHRINK, 0);
  hbox->pack_start(*tab_size, Gtk::PACK_SHRINK, 0);
  hbox2->pack_start(*label_autosave, Gtk::PACK_SHRINK, 0);
  hbox2->pack_start(*asave_minutes, Gtk::PACK_SHRINK, 0);
  hbox2->pack_start(*label_minutes, Gtk::PACK_SHRINK, 0);
  hbox3->pack_start(*label_use_terminal, Gtk::PACK_SHRINK, 0);
  hbox3->pack_start(*terminal, Gtk::PACK_SHRINK, 0);
  fixed->put(*replace_tabs, 16, 0);
  fixed2->put(*load_on_launch, 16, 0);
  vbox->set_border_width(10);
  vbox->set_spacing(10);
  vbox->pack_start(*label_tabstops, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*fixed, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*label_filemanagement, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*hbox2, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*fixed2, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*label_terminal, Gtk::PACK_SHRINK, 0);
  vbox->pack_start(*hbox3, Gtk::PACK_SHRINK, 0);
  
  label_font->set_alignment(0,0.5);
  label_compiler->set_alignment(0,0.5);
  label_error->set_padding(10,0);
  label_warning->set_padding(10,0);
  label_editor_font->set_padding(10,0);
  error_color->set_text(env.get_error_color());
  warning_color->set_text(env.get_warning_color());
  fontbutton->set_font_name(Glib::ustring(env.get_font_face() + " " + env.get_font_size()));
  button_color1->add(*image1);
  button_color2->add(*image2);
  hbox4->pack_start(*label_editor_font, Gtk::PACK_SHRINK, 0);
  hbox4->pack_start(*fontbutton, Gtk::PACK_SHRINK, 0);
  hbox5->pack_start(*label_error, Gtk::PACK_SHRINK, 0);
  hbox5->pack_start(*error_color, Gtk::PACK_SHRINK, 0);
  hbox5->pack_start(*button_color1, Gtk::PACK_SHRINK, 10);
  hbox6->pack_start(*label_warning, Gtk::PACK_SHRINK, 0);
  hbox6->pack_start(*warning_color, Gtk::PACK_SHRINK, 0);
  hbox6->pack_start(*button_color2, Gtk::PACK_SHRINK, 10);
  vbox2->set_border_width(10);
  vbox2->set_spacing(10);
  vbox2->pack_start(*label_font, Gtk::PACK_SHRINK, 0);
  vbox2->pack_start(*hbox4, Gtk::PACK_SHRINK, 0);
  vbox2->pack_start(*label_compiler, Gtk::PACK_SHRINK, 0);
  vbox2->pack_start(*hbox5, Gtk::PACK_SHRINK, 0);
  vbox2->pack_start(*hbox6, Gtk::PACK_SHRINK, 0);
  
  label_elements->set_alignment(0,0.5);
  label_vars->set_padding(10,0);
  label_cmnt->set_padding(10,0);
  label_strn->set_padding(10,0);
  label_chars->set_padding(10,0);
  label_kywd->set_padding(10,0);
  label_prep->set_padding(10,0);
  variables->set_text(env.get_syntax_color(0));
  characters->set_text(env.get_syntax_color(1));
  strings->set_text(env.get_syntax_color(2));
  comments->set_text(env.get_syntax_color(3));
  keywords->set_text(env.get_syntax_color(4));
  preprocessor->set_text(env.get_syntax_color(5));
  button_color3->add(*image3);
  button_color4->add(*image4);
  button_color5->add(*image5);
  button_color6->add(*image6);
  button_color7->add(*image7);
  button_color8->add(*image8);
  hbox7->pack_start(*label_vars, Gtk::PACK_SHRINK, 0);
  hbox7->pack_start(*variables, Gtk::PACK_SHRINK, 0);
  hbox7->pack_start(*button_color3, Gtk::PACK_SHRINK, 10);
  hbox8->pack_start(*label_chars, Gtk::PACK_SHRINK, 0);
  hbox8->pack_start(*characters, Gtk::PACK_SHRINK, 0);
  hbox8->pack_start(*button_color4, Gtk::PACK_SHRINK, 10);
  hbox9->pack_start(*label_strn, Gtk::PACK_SHRINK, 0);
  hbox9->pack_start(*strings, Gtk::PACK_SHRINK, 0);
  hbox9->pack_start(*button_color5, Gtk::PACK_SHRINK, 10);
  hbox10->pack_start(*label_cmnt, Gtk::PACK_SHRINK, 0);
  hbox10->pack_start(*comments, Gtk::PACK_SHRINK, 0);
  hbox10->pack_start(*button_color6, Gtk::PACK_SHRINK, 10);
  hbox11->pack_start(*label_kywd, Gtk::PACK_SHRINK, 0);
  hbox11->pack_start(*keywords, Gtk::PACK_SHRINK, 0);
  hbox11->pack_start(*button_color7, Gtk::PACK_SHRINK, 10);
  hbox12->pack_start(*label_prep, Gtk::PACK_SHRINK, 0);
  hbox12->pack_start(*preprocessor, Gtk::PACK_SHRINK, 0);
  hbox12->pack_start(*button_color8, Gtk::PACK_SHRINK, 10);
  vbox3->set_border_width(10);
  vbox3->set_spacing(10);
  vbox3->pack_start(*label_elements, Gtk::PACK_SHRINK, 0);
  vbox3->pack_start(*hbox7, Gtk::PACK_SHRINK, 0);
  vbox3->pack_start(*hbox8, Gtk::PACK_SHRINK, 0);
  vbox3->pack_start(*hbox9, Gtk::PACK_SHRINK, 0);
  vbox3->pack_start(*hbox10, Gtk::PACK_SHRINK, 0);
  vbox3->pack_start(*hbox11, Gtk::PACK_SHRINK, 0);
  vbox3->pack_start(*hbox12, Gtk::PACK_SHRINK, 0);

  notebook->set_scrollable(true);
  notebook->append_page(*vbox, *label_editor);
  notebook->append_page(*vbox2, *label_fonts);
  notebook->append_page(*vbox3, *label_syntax);

  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*notebook);
  dialog->set_title("Environment Settings");
  dialog->set_modal(false);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(true);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(true);
  dialog->add_action_widget(*cancel, -6);
  dialog->add_action_widget(*ok, -5);
  dialog->show_all_children();
  dialog->show();

}

void environment_dialog::initialize_objects()
{
  Pango::FontDescription fd("Sans Bold 10");
  
  notebook = new Gtk::Notebook();
  fontbutton = new Gtk::FontButton();
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  ok = new Gtk::Button(Gtk::StockID("gtk-ok"));
  button_color1 = new Gtk::Button();
  button_color2 = new Gtk::Button();
  button_color3 = new Gtk::Button();
  button_color4 = new Gtk::Button();
  button_color5 = new Gtk::Button();
  button_color6 = new Gtk::Button();
  button_color7 = new Gtk::Button();
  button_color8 = new Gtk::Button();
  
  label_tabstops = new Gtk::Label("Tab Stops");
  label_tabwidth = new Gtk::Label("  Tab Width:");
  label_terminal = new Gtk::Label("Terminal");
  label_use_terminal = new Gtk::Label("  Use terminal:");
  label_filemanagement = new Gtk::Label("File Management");
  label_autosave = new Gtk::Label("  Autosave every");
  label_minutes = new Gtk::Label("minutes");
  label_editor = new Gtk::Label("Editor");
  label_font = new Gtk::Label("Font");
  label_editor_font = new Gtk::Label("  Editor Font:");
  label_compiler = new Gtk::Label("Compiler");
  label_error = new Gtk::Label("  Error Color:     ");
  label_fonts = new Gtk::Label("Fonts & Colors");
  label_elements = new Gtk::Label("Elements");
  label_vars = new Gtk::Label("  Variables:       ");
  label_warning = new Gtk::Label("  Warning Color:");
  label_chars = new Gtk::Label("  Characters:    ");
  label_strn = new Gtk::Label("  Strings:          ");
  label_cmnt = new Gtk::Label("  Comments:   ");
  label_kywd = new Gtk::Label("  Keywords:      ");
  label_prep = new Gtk::Label("  Preprocessor:");
  label_syntax = new Gtk::Label("Syntax Highlighting");
  
  tab_size_adj = new Gtk::Adjustment(1, 0, 9, 1, 10, 10);
  asave_minutes_adj = new Gtk::Adjustment(1, 0, 99, 1, 10, 10);
  tab_size = new Gtk::SpinButton(*tab_size_adj, 1, 0);
  asave_minutes = new Gtk::SpinButton(*asave_minutes_adj, 1, 0);
  
  replace_tabs = new Gtk::CheckButton("Insert spaces instead of tabs");
  load_on_launch = new Gtk::CheckButton("Load last project on launch");
  
  hbox = new Gtk::HBox(false, 0);
  hbox2 = new Gtk::HBox(false, 0);
  hbox3 = new Gtk::HBox(false, 0);
  hbox4 = new Gtk::HBox(false, 0);
  hbox5 = new Gtk::HBox(false, 0);
  hbox6 = new Gtk::HBox(false, 0);
  hbox7 = new Gtk::HBox(false, 0);
  hbox8 = new Gtk::HBox(false, 0);
  hbox9 = new Gtk::HBox(false, 0);
  hbox10 = new Gtk::HBox(false, 0);
  hbox11 = new Gtk::HBox(false, 0);
  hbox12 = new Gtk::HBox(false, 0);
  vbox = new Gtk::VBox(false, 7);
  vbox2 = new Gtk::VBox(false, 7);
  vbox3 = new Gtk::VBox(false, 7);
  fixed = new Gtk::Fixed();
  fixed2 = new Gtk::Fixed();
  
  terminal = new Gtk::Entry();
  error_color = new Gtk::Entry();
  warning_color = new Gtk::Entry();
  variables = new Gtk::Entry();
  characters = new Gtk::Entry();
  strings = new Gtk::Entry();
  comments = new Gtk::Entry();
  keywords = new Gtk::Entry();
  preprocessor = new Gtk::Entry();
  
  image1 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image2 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image3 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image4 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image5 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image6 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image7 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  image8 = new Gtk::Image(Gtk::StockID("gtk-select-color"), Gtk::IconSize(4));
  
  label_tabstops->modify_font(fd);
  label_filemanagement->modify_font(fd);
  label_terminal->modify_font(fd);
  label_font->modify_font(fd);
  label_compiler->modify_font(fd);
  label_elements->modify_font(fd);
}

environment_dialog::~environment_dialog()
{ 
}

Glib::ustring environment_dialog::get_font_size()
{ 
  Glib::ustring font = fontbutton->get_font_name();
  while (font.find(" ", 0) != string::npos)
    font.erase(0, font.find(" ", 0)+1);
  return font;
}

Glib::ustring environment_dialog::get_font_face()
{ 
  Glib::ustring font = fontbutton->get_font_name();
  unsigned int loc = font.find(' ', 0);
  while (font.find(' ', loc+1) != string::npos)
    loc = font.find(' ', loc+1);
  font.erase(loc, font.length()-loc);
  return font;
}

int  environment_dialog::get_asave_minutes()  { return static_cast<int>(asave_minutes->get_value()); }
int  environment_dialog::get_tab_size()       { return static_cast<int>(tab_size->get_value()); }
bool environment_dialog::get_load_on_launch() { return load_on_launch->get_active(); }
bool environment_dialog::get_replace_tabs()   { return replace_tabs->get_active(); }

Glib::ustring environment_dialog::get_terminal()      { return terminal->get_text(); }
Glib::ustring environment_dialog::get_error_color()   { return error_color->get_text(); }
Glib::ustring environment_dialog::get_warning_color() { return warning_color->get_text(); }
Glib::ustring environment_dialog::get_variables()     { return variables->get_text(); }
Glib::ustring environment_dialog::get_characters()    { return characters->get_text(); }
Glib::ustring environment_dialog::get_strings()       { return strings->get_text(); }
Glib::ustring environment_dialog::get_comments()      { return comments->get_text(); }
Glib::ustring environment_dialog::get_keywords()      { return keywords->get_text(); }
Glib::ustring environment_dialog::get_preprocessor()  { return preprocessor->get_text(); }
