/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "menu.h"
#include "utils.h"

Glib::ustring create_toolbar_string(Glib::ustring);

// Create new IconFactory
void openldev::create_icon_factory()
{
  Glib::RefPtr<Gtk::IconFactory> icon_factory = Gtk::IconFactory::create();
  
  // Add new stock menu items
  add_new_stock_item(icon_factory, "Compile", "COMPILE", "/usr/share/pixmaps/openldev/compile_sm.png");
  add_new_stock_item(icon_factory, "Build", "BUILD_SM", "/usr/share/pixmaps/openldev/build_sm.png");
  add_new_stock_item(icon_factory, "Build", "BUILD_LG", "/usr/share/pixmaps/openldev/build_lg.png");
  add_new_stock_item(icon_factory, "Project Files", "PROJFILES_SM", "/usr/share/pixmaps/openldev/edit_sm.png");
  add_new_stock_item(icon_factory, "Project Files", "PROJFILES_LG", "/usr/share/pixmaps/openldev/edit_lg.png");
  add_new_stock_item(icon_factory, "About", "ABOUT", "/usr/share/pixmaps/openldev/about.png");
  
  // Add items to default
  icon_factory->add_default();
}

void openldev::create_action_group()
{
  // Create action group
  actgroup = Gtk::ActionGroup::create();
    
  // Create File menu items
  actgroup->add(Gtk::Action::create("FileMenu", "_File"));
  actgroup->add(Gtk::Action::create("NewFileMenu", Gtk::Stock::NEW, "_New File", "Create a new file"), 
    Gtk::AccelKey("<control>n"), sigc::mem_fun(*this, &openldev::on_file_new_file));
  actgroup->add(Gtk::Action::create("NF", Gtk::Stock::NEW, "_New", 
    "Create a new file"), sigc::mem_fun(*this, &openldev::on_file_new_file));
  actgroup->add(Gtk::Action::create("NP", Gtk::Stock::NEW, "New Project", "Create a new project"), 
    Gtk::AccelKey("<control>p"), sigc::mem_fun(*this, &openldev::on_file_new));
  actgroup->add(Gtk::Action::create("OP", Gtk::Stock::OPEN, "_Open",
    "Open a file or project"), sigc::mem_fun(*this, &openldev::on_file_open));
  actgroup->add(Gtk::Action::create("SV", Gtk::Stock::SAVE, "_Save",
    "Save current file"), sigc::mem_fun(*this, &openldev::on_file_save));
  actgroup->add(Gtk::Action::create("SA", Gtk::Stock::SAVE_AS, "Save _As ...",
    "Save current file by a new name"), sigc::mem_fun(*this, &openldev::on_file_saveas));
  actgroup->add(Gtk::Action::create("SaveAll", "Save All"),
    sigc::mem_fun(*this, &openldev::on_file_saveall));
  actgroup->add(Gtk::Action::create("CloseProj", "Close Project"), 
    sigc::mem_fun(*this, &openldev::close_project));
  actgroup->add(Gtk::Action::create("QT", Gtk::Stock::QUIT, "_Quit",
    "Quit the application"), sigc::mem_fun(*this, &openldev::on_file_quit));
  
  // Create Edit menu items
  actgroup->add(Gtk::Action::create("EditMenu", "_Edit"));
  actgroup->add(Gtk::Action::create("UD", Gtk::Stock::UNDO, "_Undo", "Undo the last action"),
    Gtk::AccelKey("<control>z"), sigc::mem_fun(*this, &openldev::on_edit_undo));
  actgroup->add(Gtk::Action::create("RD", Gtk::Stock::REDO, "_Redo", "Redo the next action"),
    Gtk::AccelKey("<control>y"), sigc::mem_fun(*this, &openldev::on_edit_redo));
  actgroup->add(Gtk::Action::create("CT", Gtk::Stock::CUT, "_Cut", "Cut the selected text"),
    sigc::mem_fun(*this, &openldev::on_edit_cut));
  actgroup->add(Gtk::Action::create("CO", Gtk::Stock::COPY, "_Copy", "Copy the selected text"),
    sigc::mem_fun(*this, &openldev::on_edit_copy));
  actgroup->add(Gtk::Action::create("PT", Gtk::Stock::PASTE, "_Paste", "Paste text from the clipboard"),
    sigc::mem_fun(*this, &openldev::on_edit_paste));
  actgroup->add(Gtk::Action::create("SelectAll", "Select _All\t"),
    Gtk::AccelKey("<control>a"), sigc::mem_fun(*this, &openldev::on_edit_selectall));
  actgroup->add(Gtk::Action::create("Deselect", "_Deselect"),
    sigc::mem_fun(*this, &openldev::on_edit_deselect));
  actgroup->add(Gtk::Action::create("FD", Gtk::Stock::FIND, "_Find", "Find text in document"),
    sigc::mem_fun(*this, &openldev::on_edit_find));
  actgroup->add(Gtk::Action::create("FN", Gtk::Stock::FIND, "Find Next", "Find text in document"),
    Gtk::AccelKey("F3"), sigc::mem_fun(*this, &openldev::on_find_next));
  actgroup->add(Gtk::Action::create("GL", Gtk::Stock::JUMP_TO, "_Goto Line", "Goto line number"),
    Gtk::AccelKey("<control>g"), sigc::mem_fun(*this, &openldev::on_edit_goto_line));
  
  // Create View menu items
  actgroup->add(Gtk::Action::create("ViewMenu", "_View"));
  toggle_lnumbers = Gtk::ToggleAction::create("LineNumbers", "Line Numbering",
    "Show line numbering", env.get_line_numbers());
  toggle_status = Gtk::ToggleAction::create("StatusBar", "Status Bar",
    "Show status bar", true);
  toggle_toolbar = Gtk::ToggleAction::create("Toolbar", "Main Toolbar",
    "Show main toolbar", true);
  actgroup->add(toggle_toolbar, sigc::mem_fun(*this, &openldev::on_view_toolbar));
  actgroup->add(toggle_status, sigc::mem_fun(*this, &openldev::on_view_statusbar));
  actgroup->add(toggle_lnumbers, sigc::mem_fun(*this, &openldev::on_view_lnumbers));
  actgroup->add(Gtk::Action::create("FileView", Gtk::Stock::JUSTIFY_LEFT, "Project Files", "Show project file browser"),
    sigc::mem_fun(*this, &openldev::on_view_fileviewer));
  actgroup->add(Gtk::Action::create("BuildOutput", Gtk::Stock::JUSTIFY_LEFT, "Build Output", "Show build output browser"),
    sigc::mem_fun(*this, &openldev::on_view_buildbox));
  
  // Create Project menu items
  actgroup->add(Gtk::Action::create("ProjectMenu", "_Project"));
  actgroup->add(Gtk::Action::create("PI",Gtk::Stock::PROPERTIES, "Project Info", "Project information"),
    sigc::mem_fun(*this, &openldev::on_project_properties));
  actgroup->add(Gtk::Action::create("CP", Gtk::Stock::PREFERENCES, "Configure Project", "Configure project properties"),
    sigc::mem_fun(*this, &openldev::on_settings_file_properties));
  actgroup->add(Gtk::Action::create("AF", Gtk::Stock::ADD, "Add Files", "Add existing file"),
    sigc::mem_fun(*this, &openldev::tree_view_add_file));
  actgroup->add(Gtk::Action::create("RF", Gtk::Stock::REMOVE, "Remove File", "Remove selected file"),
    sigc::mem_fun(*this, &openldev::tree_view_remove));
  
  // Create Build menu items
  actgroup->add(Gtk::Action::create("BuildMenu", "_Build"));
  actgroup->add(Gtk::Action::create("CL", Gtk::StockID("COMPILE"), "_Compile", "Compile current file"),
    Gtk::AccelKey("F9"), sigc::mem_fun(*this, &openldev::on_compile));
  actgroup->add(Gtk::Action::create("Build", Gtk::StockID("BUILD_SM"), "_Build", "Build the project"),
    Gtk::AccelKey("F11"), sigc::mem_fun(*this, &openldev::on_build));
  actgroup->add(Gtk::Action::create("BU", Gtk::StockID("BUILD_LG"), "_Build", "Build the project"),
    sigc::mem_fun(*this, &openldev::on_build));
  actgroup->add(Gtk::Action::create("EX", Gtk::Stock::EXECUTE, "_Execute", "Execute the program"),
    Gtk::AccelKey("F4"), sigc::mem_fun(*this, &openldev::on_execute));
  actgroup->add(Gtk::Action::create("AG", Gtk::Stock::CONVERT, "Auto _Generate", "Generate build files & Run configure"),
    sigc::mem_fun(*this, &openldev::on_autogen));
  actgroup->add(Gtk::Action::create("Configure", "Configure"),
    sigc::mem_fun(*this, &openldev::on_configure));
  actgroup->add(Gtk::Action::create("MC", Gtk::Stock::CLEAR, "_Make Clean", "Clean project directories"),
    sigc::mem_fun(*this, &openldev::on_clean));
  actgroup->add(Gtk::Action::create("ST", Gtk::Stock::STOP, "_Stop", "Stop the current build"),
    Gtk::AccelKey("F12"), sigc::mem_fun(*this, &openldev::on_stop));
  
  // Create Settings menu items
  actgroup->add(Gtk::Action::create("SettingsMenu", "_Settings"));
  actgroup->add(Gtk::Action::create("EN", Gtk::Stock::PREFERENCES, "_Environment", "Environment settings"),
    sigc::mem_fun(*this, &openldev::on_settings_env));
  actgroup->add(Gtk::Action::create("CustomizeToolbar", "_Customize Toolbar"),
    sigc::mem_fun(*this, &openldev::on_settings_toolbar));
  actgroup->add(Gtk::Action::create("Files", Gtk::StockID("PROJFILES_SM"), "Project Files", "Project file manager"),
    sigc::mem_fun(*this, &openldev::on_settings_filemanager));
  actgroup->add(Gtk::Action::create("PF", Gtk::StockID("PROJFILES_LG"), "Project Files", "Project file manager"),
    sigc::mem_fun(*this, &openldev::on_settings_filemanager));
  
  // Create Help menu items
  actgroup->add(Gtk::Action::create("HelpMenu", "_Help"));
  actgroup->add(Gtk::Action::create("HP", Gtk::Stock::HELP),
    Gtk::AccelKey("F1"), sigc::mem_fun(*this, &openldev::on_help));
  actgroup->add(Gtk::Action::create("Bug", "Report Bugs"),
    sigc::mem_fun(*this, &openldev::on_report_bug));
  actgroup->add(Gtk::Action::create("About", Gtk::StockID("ABOUT"), "About", "About OpenLDev"),
    Gtk::AccelKey("<alt>a"), sigc::mem_fun(*this, &openldev::on_help_about));
  
  // Create popup menu items
  actgroup->add(Gtk::Action::create("TreeViewMenu", "Tree View Menu"));
  actgroup->add(Gtk::Action::create("OpenFile", Gtk::Stock::OPEN),
    sigc::mem_fun(*this, &openldev::tree_view_selected));
  actgroup->add(Gtk::Action::create("AddFile", Gtk::Stock::ADD, "Add File", "Add existing file"),
    sigc::mem_fun(*this, &openldev::tree_view_add_file));
  actgroup->add(Gtk::Action::create("DeleteFile", Gtk::Stock::REMOVE),
    sigc::mem_fun(*this, &openldev::tree_view_remove));
}

// Create UIManager information string
void openldev::create_uimanager_string()
{
  ui_info = 
     "<ui>"
     "  <menubar name='MenuBar'>"
     "    <menu action='FileMenu'>"
     "      <menuitem action='NewFileMenu'/>"
     "      <menuitem action='NP'/>"
     "      <menuitem action='OP'/>"
     "      <separator/>"
     "      <menuitem action='SV'/>"
     "      <menuitem action='SA'/>"
     "      <menuitem action='SaveAll'/>"
     "      <separator/>"
     "      <menuitem action='CloseProj'/>"
     "      <menuitem action='QT'/>"
     "    </menu>"
     "    <menu action='EditMenu'>"
     "      <menuitem action='UD'/>"
     "      <menuitem action='RD'/>"
     "      <separator/>"
     "      <menuitem action='CT'/>"
     "      <menuitem action='CO'/>"
     "      <menuitem action='PT'/>"
     "      <separator/>"
     "      <menuitem action='SelectAll'/>"
     "      <menuitem action='Deselect'/>"
     "      <separator/>"
     "      <menuitem action='FD'/>"
     "      <menuitem action='FN'/>"
     "      <menuitem action='GL'/>"
     "    </menu>"
     "    <menu action='ViewMenu'>"
     "      <menuitem action='FileView'/>"
     "      <menuitem action='BuildOutput'/>"
     "      <separator/>"
     "      <menuitem action='Toolbar'/>"
     "      <menuitem action='StatusBar'/>"
     "      <separator/>"
     "      <menuitem action='LineNumbers'/>"
     "    </menu>"
     "    <menu action='ProjectMenu'>"
     "      <menuitem action='AF'/>"
     "      <menuitem action='RF'/>"
     "      <separator/>"
     "      <menuitem action='Files'/>"
     "      <menuitem action='CP'/>"
     "      <separator/>"
     "      <menuitem action='PI'/>"
     "    </menu>"
     "    <menu action='BuildMenu'>"
     "      <menuitem action='CL'/>"
     "      <menuitem action='Build'/>"
     "      <menuitem action='EX'/>"
     "      <separator/>"
     "      <menuitem action='AG'/>"
     "      <menuitem action='Configure'/>"
     "      <menuitem action='MC'/>"
     "      <separator/>"
     "      <menuitem action='ST'/>"
     "    </menu>"
     "    <menu action='SettingsMenu'>"
     "      <menuitem action='EN'/>"
     "      <menuitem action='CustomizeToolbar'/>"
     "    </menu>"
     "    <menu action='HelpMenu'>"
     "      <menuitem action='HP'/>"
     "      <menuitem action='Bug'/>"
     "      <separator/>"
     "      <menuitem action='About'/>"
     "    </menu>"
     "  </menubar>"
     "  <popup name='TreeViewMenu'>"
     "    <menuitem action='AddFile'/>"
     "    <menuitem action='OpenFile'/>"
     "    <menuitem action='DeleteFile'/>"
     "    <separator/>"
     "    <menuitem action='CP'/>"
     "  </popup>"
     "</ui>";
     
  ui_toolbar = "<toolbar name='ToolBar'>"; 
  ui_toolbar += create_toolbar_string(env.get_toolbar());
  ui_toolbar += "</toolbar>";
}

Glib::ustring create_toolbar_string(Glib::ustring tb_str)
{
  Glib::ustring str;
  
  while (!tb_str.empty())
  {
    if (tb_str.substr(0, 2) == "SP")
    {
      str += "<separator/>";
    }
    else
    {
      str += "<toolitem action='";
      str += tb_str.substr(0, 2);
      str += "'/>";
    }
    tb_str.erase(0, 2);
  }
  
  return str;
}
