/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "new.h"
#include "utils.h"
#include <gtkmm/treemodel.h>
#include <gtkmm/liststore.h>
#include <gtkmm/messagedialog.h>
#include <glib.h>
#include <sys/stat.h>
#include <cstdio>

// New file dialog
new_project_dialog::new_project_dialog ()
{
  dialog = this;
  initialize_objects();
  
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  dialog->set_default_icon_from_file("/usr/share/pixmaps/openldev/openldev24.png");
  populate_combobox();
  
  combobox->set_size_request(230,30);
  name->set_size_request(230,25);
  version->set_size_request(230,25);
  homepage->set_size_request(230,25);
  email->set_size_request(230,25);
  author->set_size_request(230,25);
  
  // Set widgets to the Fixed box
  hbox3->set_border_width(10);
  hbox3->pack_start(*label, Gtk::PACK_SHRINK, 10);
  hbox3->pack_start(*combobox, Gtk::PACK_EXPAND_WIDGET, 10);
  
  // Create the labels for the file/project properties
  location->set_text(Glib::ustring(Glib::get_home_dir() + "/openldev/"));
  label1->set_alignment(1.0,0.5);
  label2->set_alignment(1.0,0.5);
  label3->set_alignment(1.0,0.5);
  label4->set_alignment(1.0,0.5);
  label5->set_alignment(1.0,0.5);
  label6->set_alignment(1.0,0.5);
  
  // Pack the browse & location in HBox2
  hbox2->set_spacing(5);
  hbox2->pack_start(*location);
  hbox2->pack_start(*browse, Gtk::PACK_SHRINK);

  // Add labels & entries to the H/VBox "table"
  vbox2->set_spacing(3);
  vbox2->pack_start(*label5, Gtk::PACK_SHRINK, 13);
  vbox2->pack_start(*label6, Gtk::PACK_SHRINK, 13);
  vbox2->pack_start(*label4, Gtk::PACK_SHRINK, 13);
  vbox2->pack_start(*label3, Gtk::PACK_SHRINK, 13);
  vbox2->pack_start(*label2, Gtk::PACK_SHRINK, 13);
  vbox2->pack_start(*label1, Gtk::PACK_SHRINK, 11);
  vbox3->pack_start(*name, Gtk::PACK_SHRINK, 10);
  vbox3->pack_start(*hbox2, Gtk::PACK_SHRINK, 10);
  vbox3->pack_start(*author, Gtk::PACK_SHRINK, 10);
  vbox3->pack_start(*email, Gtk::PACK_SHRINK, 10);
  vbox3->pack_start(*homepage, Gtk::PACK_SHRINK, 10);
  vbox3->pack_start(*version, Gtk::PACK_SHRINK, 10);
  hbox4->pack_start(*vbox2, Gtk::PACK_SHRINK, 10);
  hbox4->pack_start(*vbox3, Gtk::PACK_EXPAND_WIDGET, 10);
  
  // Create & pack the buttons
  cancel->set_size_request(85, 30);
  apply->set_size_request(85, 30);
  hbox->pack_end(*apply, Gtk::PACK_SHRINK, 6);
  hbox->pack_end(*cancel, Gtk::PACK_SHRINK, 6);
  
  // Add widgets to the overall vbox
  vbox->set_spacing(5);
  vbox->pack_start(*hbox3, Gtk::PACK_SHRINK);
  vbox->pack_start(*hseparator, Gtk::PACK_SHRINK);
  vbox->pack_start(*hbox4);
  vbox->pack_start(*hseparator2, Gtk::PACK_SHRINK, 3);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK);
  
  // Set the dialog properties
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*vbox);
  dialog->set_size_request(350, 400);
  dialog->set_title("New Project");
  dialog->set_modal(true);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(false);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(false);

  dialog->show_all_children();
  dialog->show();
  
  apply->signal_clicked().connect(sigc::mem_fun(*this, &new_project_dialog::on_apply));
  cancel->signal_clicked().connect(sigc::mem_fun(*this, &new_project_dialog::on_cancel));
  browse->signal_clicked().connect(sigc::mem_fun(*this, &new_project_dialog::on_browse));
}

void new_project_dialog::initialize_objects()
{
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  apply = new Gtk::Button(Gtk::StockID("gtk-ok"));
  browse = new Gtk::Button("...");
  hseparator = new Gtk::HSeparator();
  hseparator2 = new Gtk::HSeparator();
  combobox = new Gtk::Combo();
  
  vbox = new Gtk::VBox(false, 0);
  vbox2 = new Gtk::VBox(false, 0);
  vbox3 = new Gtk::VBox(false, 0);
  hbox = new Gtk::HBox(false, 0);
  hbox2 = new Gtk::HBox(false, 0);
  hbox3 = new Gtk::HBox(false, 0);
  hbox4 = new Gtk::HBox(false, 0);
  
  version = new Gtk::Entry();
  homepage = new Gtk::Entry();
  location = new Gtk::Entry();
  email = new Gtk::Entry();
  author = new Gtk::Entry();
  name = new Gtk::Entry();
  
  label = new Gtk::Label("Create New:");
  label1 = new Gtk::Label("Version:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label2 = new Gtk::Label("Homepage:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label3 = new Gtk::Label("E-mail:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label4 = new Gtk::Label("Author:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label5 = new Gtk::Label("Name:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  label6 = new Gtk::Label("Directory:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
}

new_project_dialog::~new_project_dialog()
{
  delete cancel;
  delete apply;
  delete browse;
  delete label;
  delete label1;
  delete label2;
  delete label3;
  delete label4;
  delete label5;
  delete label6;
  delete vbox;
  delete hbox;
  delete hbox2;
  delete name;
  delete author;
  delete email;
  delete homepage;
  delete version;
  delete location;
  delete hbox3;
  delete combobox;
  delete hseparator;
  delete hseparator2;
}

// Call this when cancel is pressed
void new_project_dialog::on_cancel(void)
{
  dialog->response(Gtk::RESPONSE_CANCEL);
  dialog->hide();
}

// Let the user choose a location for their project
void new_project_dialog::on_browse(void)
{
  Gtk::FileChooserDialog dialog("Choose Directory", Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  
  int result = dialog.run();
  switch (result)
  {
    // If OK is pressed
    case (Gtk::RESPONSE_OK):
    {
      location->set_text(Glib::ustring(dialog.get_filename() + "/"));
      break;
    }
    default:
      break;
  }
}

// When the forward button is pressed ...
void new_project_dialog::on_apply(void)
{
  bool test = true;
  Gtk::Entry *choice = combobox->get_entry();
  Glib::ustring choice_text = choice->get_text(), errortext;
  
  Glib::ustring ntxt = name->get_text(), vtxt = version->get_text(),
                ltxt = location->get_text();
  
  // Check for error when creating a project or file
  if (ntxt.find(" ",0) != Glib::ustring::npos ||
      ntxt.find("_",0) != Glib::ustring::npos ||
      ntxt.empty())
  {
    errortext = "The project name must\nbe alphanumeric!";
    test = false;
  }
  else if (vtxt.empty())
  {
    errortext = "You must specify a\nversion number!";
    test = false;
  }
  else if (ltxt.empty())
  {
    errortext = "You must specify a\nproject location!";
    test = false;
  }
  
  // Create new project
  if (test)
  {
    FILE *command;
    Glib::ustring odp_file_text, odp_file_location, project_path, copy, extract,
                  rm, configure_in, makefile_am, makefile_am2, text;
    unsigned int loc;
    
    // Add a '/' to the end of the location if it does not exist
    if (location->get_text().at(location->get_text().length()-1) != '/')
    {
      Glib::ustring str = location->get_text();
      str += "/";
      location->set_text(str);
    }
    
    // Create /home/user/openldev/ directory if it does not exist and it
    //  is the chosen location
    if (location->get_text() == Glib::ustring(Glib::get_home_dir() + "/openldev/"))
    {
      Glib::ustring dir = Glib::get_home_dir();
      dir += "/openldev/";
      mkdir(dir.c_str(), 0777);
    }
    
    // Create .ODP file text
    odp_file_text = "<name>" + name->get_text() + "*\n<author>" + author->get_text()
      + "*\n<location>" + location->get_text() + name->get_text()
      + "/*\n<email>" + email->get_text() + "*\n<url>" + homepage->get_text() 
      + "*\n<version>" + version->get_text();
    if (choice_text == "Terminal Project (C)")
      odp_file_text += "*\n<language>C*\n<lastfile>main.c*\n<sources>main.c/*\n<headers>*\n<execname>";
    else
      odp_file_text += "*\n<language>+*\n<lastfile>main.cc*\n<sources>main.cc/*\n<headers>*\n<execname>";
    
    // Create .openldev file location
    odp_file_location = location->get_text() + name->get_text()
      + "/" + name->get_text() + ".openldev";
    
    // Create directory path
    project_path += location->get_text() + name->get_text() + "/";
    
    // Create copy command, configure.in path, & makefile.am path
    if (choice_text == "Terminal Project (C)") 
    {
      copy = "cp -rf /usr/share/openldev/templates/console-c.tar ";
      extract = "tar -xf console-c.tar";
      rm = "rm -f console-c.tar";
    } 
    else 
    {
      copy = "cp -rf /usr/share/openldev/templates/console-cxx.tar ";
      extract = "tar -xf console-cxx.tar";
      rm = "rm -f console-cxx.tar";
    }
    copy += project_path;  
    configure_in = project_path + "configure.in";
    makefile_am = project_path + "src/Makefile.am";
    makefile_am2 = project_path + "Makefile.am";
    
    // Make project directory & copy files over
    mkdir(project_path.c_str(), 0777);
    command = popen(copy.c_str(), "r");
    pclose(command);
    
    // Extract & delete file
    chdir(project_path.c_str());
    command = popen(extract.c_str(), "r");
    pclose(command);
    command = popen(rm.c_str(), "r");
    pclose(command);
    
    // Create project directory & store project file location in temp file
    write_file(odp_file_location, odp_file_text);
    file_location = odp_file_location;
    
    // Edit configure.in file
    text = read_file(configure_in);
    loc = text.find("TITLE",0);
    text.erase(loc,5);
    text.insert(loc,name->get_text());
    loc = text.find("VERSION",0);
    text.erase(loc,7);
    text.insert(loc,version->get_text());
    write_file(configure_in, text);
      
    // Edit srcMakefile.am file
    text = read_file(makefile_am);
    loc = text.find("TITLE",0);
    while (loc != Glib::ustring::npos)
    {
      text.erase(loc,5);
      text.insert(loc,name->get_text());
      loc = text.find("TITLE",0);
    }
    write_file(makefile_am, text);
    
    // Edit Makefile.am file
    text = read_file(makefile_am2);
    loc = text.find("TITLE",0);
    while (loc != Glib::ustring::npos)
    {
      text.erase(loc,5);
      text.insert(loc,name->get_text());
      loc = text.find("TITLE",0);
    }
    write_file(makefile_am2, text);
     
    dialog->response(Gtk::RESPONSE_ACCEPT);
    dialog->hide();
  }
  // Throw an error
  else 
    error(errortext);
}

// Called when Project or File radio button is chosen. This re-populates the treeview
void new_project_dialog::populate_combobox()
{
  list<Glib::ustring> types;
    
  types.push_back("Terminal Project (C)");
  types.push_back("Terminal Project (C++)");
  
  combobox->set_popdown_strings(types);
}

Glib::ustring new_project_dialog::get_filename() { return file_location; }
