/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "newfile.h"
#include "utils.h"
#include <gtkmm/treemodel.h>
#include <gtkmm/liststore.h>
#include <gtkmm/messagedialog.h>
#include <glib.h>
#include <sys/stat.h>
#include <cstdio>

// New file dialog
new_file_dialog::new_file_dialog (project_settings proj_settings)
{
  file_location = proj_settings.get_location() + "src/";
  
  dialog = this;
  initialize_objects();
  
  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  dialog->set_default_icon_from_file("/usr/share/pixmaps/openldev/openldev24.png");
  
  populate_combobox();
  combobox->set_size_request(230,30);
  
  // Create the labels for the file/project properties
  label1->set_alignment(1.0,0.5);
  label2->set_alignment(1.0,0.5);
  
  // Pack main widgets
  vbox2->pack_start(*label1, Gtk::PACK_SHRINK, 10);
  vbox2->pack_start(*label2, Gtk::PACK_SHRINK, 10);
  vbox3->pack_start(*combobox, Gtk::PACK_SHRINK, 5);
  vbox3->pack_start(*name, Gtk::PACK_SHRINK, 5);
  hbox2->pack_start(*vbox2, Gtk::PACK_SHRINK, 5);
  hbox2->pack_start(*vbox3, Gtk::PACK_SHRINK, 5);
  
  // Create & pack the buttons
  cancel->set_size_request(85, 30);
  apply->set_size_request(85, 30);
  hbox->pack_end(*apply, Gtk::PACK_SHRINK, 6);
  hbox->pack_end(*cancel, Gtk::PACK_SHRINK, 6);
  
  // Add widgets to the overall vbox
  vbox->set_spacing(5);
  vbox->set_border_width(5);
  vbox->pack_start(*hbox2);
  vbox->pack_start(*hseparator, Gtk::PACK_SHRINK, 3);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK, 0);
  
  // Set the dialog properties
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*vbox);
  dialog->set_title("New File");
  dialog->set_modal(true);
  dialog->property_window_position().set_value(Gtk::WIN_POS_NONE);
  dialog->set_resizable(false);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(false);

  dialog->show_all_children();
  dialog->show();
  
  apply->signal_clicked().connect(sigc::mem_fun(*this, &new_file_dialog::on_apply));
  cancel->signal_clicked().connect(sigc::mem_fun(*this, &new_file_dialog::on_cancel));
}

void new_file_dialog::initialize_objects()
{
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  apply = new Gtk::Button(Gtk::StockID("gtk-ok"));
  combobox = new Gtk::Combo();
  hseparator = new Gtk::HSeparator();
  name = new Gtk::Entry();
  hbox = new Gtk::HBox(false, 0);
  vbox = new Gtk::VBox(false, 0);
  hbox2 = new Gtk::HBox(false, 0);
  vbox2 = new Gtk::VBox(false, 0);
  vbox3 = new Gtk::VBox(false, 0);
  label1 = new Gtk::Label("Create New:");
  label2 = new Gtk::Label("File Name:");
}

new_file_dialog::~new_file_dialog()
{
  delete cancel;
  delete apply;
  delete label1;
  delete label2;
  delete name;
  delete hbox;
  delete vbox;
  delete combobox;
  delete hseparator;
}

// Call this when cancel is pressed
void new_file_dialog::on_cancel(void)
{
  dialog->response(Gtk::RESPONSE_CANCEL);
  dialog->hide();
}

// When the forward button is pressed ...
void new_file_dialog::on_apply(void)
{
  bool test = true;
  Gtk::Entry *choice = combobox->get_entry();
  Glib::ustring choice_text = choice->get_text();
  
  Glib::ustring ntxt = name->get_text();
  
  // Check for error when creating a file
  if (ntxt.find(" ",0) != Glib::ustring::npos ||
      ntxt.find("_",0) != Glib::ustring::npos ||
      ntxt.empty())
  {
    test = false;
  }
  
  // Create new file
  if (test)
  {
    Glib::ustring ext1, ext2;
    file_location += name->get_text();
    
    // Create possible file extensions
    if (choice_text == "C Source File (.c)") 
    {
      ext1 = ".c";
      ext2 = ".c";
    } 
    else if (choice_text == "C/C++ Header File (.h)") 
    {
      ext1 = ".h";
      ext2 = ".h";
    } 
    else 
    {
      ext1 = ".cc";
      ext2 = ".cpp";
    }
    
    // Add extension if it is not there
    if (name->get_text().find(ext1,0) == Glib::ustring::npos && 
        name->get_text().find(ext2,0) == Glib::ustring::npos)
      file_location += ext1;
    
    // Write new file
    write_file(file_location, "");
    
    dialog->response(Gtk::RESPONSE_ACCEPT);
    dialog->hide();
  }
  // Throw an error
  else 
    error("You must specify a\nfile name!");
}

// Called when Project or File radio button is chosen. This re-populates
//  the treeview
void new_file_dialog::populate_combobox()
{
  list<Glib::ustring> types;
  
  types.push_back("C Source File (.c)");
  types.push_back("C/C++ Header File (.h)");
  types.push_back("C++ Source File (.cc)");

  combobox->set_popdown_strings(types);
}

Glib::ustring new_file_dialog::get_filename() { return file_location; }
