/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GTKMM_MAIN_H
#define GTKMM_MAIN_H

// The motherload of includes for all other files. (Note: This file is included
// by most others in this code, so most Gtk files will already be included)
#include "environment.h"
#include "editor.h"
#include "project.h"
#include "notebookmanager.h"

#include <gtkmm/window.h>
#include <gtkmm/paned.h>
#include <gtkmm/box.h>
#include <gtkmm/menubar.h>
#include <gtkmm/menu.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/toolbar.h>
#include <gtkmm/textview.h>
#include <gtkmm/textbuffer.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/fixed.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/actiongroup.h>
#include <glib.h>
#include <iostream>
using namespace std;

const char *const clipboard = "";

// Class to add column to a TreeView (TreeModel)
class ModelColumns : public Gtk::TreeModel::ColumnRecord
{
public:
  ModelColumns()  { add(col_text); }
  Gtk::TreeModelColumn<Glib::ustring> col_text;
};

// Class to add column to a TreeView (TreeModel)
class BuildColumns : public Gtk::TreeModel::ColumnRecord
{
public:
  BuildColumns()  { add(col_text); add(col_color); }
  Gtk::TreeModelColumn<Glib::ustring> col_text, col_color;
};

// Main class for openldev
class openldev : public Gtk::Window
{
public:
  openldev();                         
  virtual ~openldev();                
  void populate_window();             

  // File commands
  void on_file_new();  
  void on_file_new_file();
  void on_file_open();                
  void on_file_save();                
  void on_file_saveas();
  void on_file_saveall();
  void on_file_quit();
  void close_project();
  void open_file(Glib::ustring);
  void open_project(Glib::ustring);
  void tree_view_selected();
  void tree_view_add_file();
  void tree_view_remove();

  // Edit commands
  void on_edit_undo();
  void on_edit_redo();
  void on_edit_cut();       
  void on_edit_copy();           
  void on_edit_paste();           
  void on_edit_selectall();
  void on_edit_deselect();
  void on_edit_find();
  void on_find_next();
  void on_find_previous();
  void on_edit_goto_line();
  void jump_to_line_number();
  void populate_find_toolbar();
  void populate_goto_line_toolbar();
  void mark_found_text(Gtk::TextIter, Gtk::TextIter);
  
  // View commands
  void on_view_toolbar();
  void on_view_statusbar();
  void on_view_fileviewer();
  void on_view_buildbox();
  void on_view_lnumbers();
  
  // Project building & compiling commands
  void on_compile();
  void on_build();
  void on_execute();
  void on_autogen();
  void on_configure();
  void on_clean();
  void on_stop();
  
  // Settings dialogs functions
  void on_settings_env();
  void on_settings_file_properties();
  void on_settings_filemanager();
  void on_settings_toolbar();
  void get_settings();
  void on_project_properties();
  
  // Help commands
  void on_help();
  void on_report_bug();
  void on_help_about();            
  
  // Utilities
  void initialize_objects();
  void populate_treeview();
  void build_box_selected(GdkEventButton*);
  bool on_keystroke(GdkEventKey*);
  void on_button_press_event_treeview(GdkEventButton*);
  void create_icon_factory();
  void create_action_group();
  void update_status_bar (Gtk::TextIter, Glib::RefPtr<Gtk::TextBuffer::Mark>);
  void create_uimanager_string();
  void on_resize(GdkEvent*);
  void on_resize_pane(GdkEvent*);
 
private:
  // Container Widgets
  Gtk::HPaned *hpaned;
  Gtk::VPaned *vpaned;
  Gtk::VBox *vbox;
  Gtk::HBox *hbox, *hbox_goto, *hbox_find;

  // Many types of bars
  Gtk::Statusbar *status_bar, *status_bar2;
  Gtk::Widget *Menubar;
  Gtk::Toolbar *toolbar, *buildbar;
  Glib::RefPtr<Gtk::UIManager> uimanager, uimanager_toolbar;
  Glib::RefPtr<Gtk::ActionGroup> actgroup;
  Glib::RefPtr<Gtk::ToggleAction> toggle_toolbar, toggle_lnumbers, toggle_status;
  Glib::ustring ui_info, ui_toolbar;
  Gtk::Menu *menu_source_browser, *menu_text_view;
  
  // Scrolled windows for files view, build box, & text view
  Gtk::ScrolledWindow *scrolled_win, *scrolled_win2;

  // Various views in the main window
  Gtk::TreeView *tree_view, *build_box;

  // Find toolbar widgets
  Gtk::Label *label_find, *label_error;
  Gtk::Entry *find_text;
  Gtk::Button *button_find, *button_find_prev, *button_close;
  Gtk::Image *image_close, *image_error;
  
  // Goto Line toolbar widgets
  Gtk::Label *label_goto_line, *label_error2;
  Gtk::Button *button_goto_line, *button_close_goto;
  Gtk::Image *image_error2, *image_close2;
  Gtk::SpinButton *goto_line;
  
  // Environment & project settings
  project_settings proj;
  environment_settings env;
  editor_properties screen;
  notebook_manager *noteman;
};

#endif
