/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "project_properties.h"
#include "utils.h"

#include <glib.h>

// Project properties dialog constructor
project_properties_dialog::project_properties_dialog (project_settings &proj)
{
  // Create Makefile.am location string
  makefile_location = proj.get_location() + "src/Makefile.am";
  initialize_objects();

  dialog->get_action_area()->property_layout_style().set_value(Gtk::BUTTONBOX_END);
  dialog->set_default_icon_from_file("/usr/share/pixmaps/openldev/openldev24.png");
  ok->set_flags(Gtk::CAN_DEFAULT);
  
  // Create entries for project properties
  name->set_text(proj.get_name());
  author->set_text(proj.get_author());
  email->set_text(proj.get_email());
  homepage->set_text(proj.get_url());
  version->set_text(proj.get_version());

  // Place labels & entries in the boxes
  name->set_editable(false);
  vbox2->pack_start(*l1);
  vbox2->pack_start(*l2);
  vbox2->pack_start(*l5);
  vbox2->pack_start(*l3);
  vbox2->pack_start(*l4);
  vbox3->pack_start(*name);
  vbox3->pack_start(*version);
  vbox3->pack_start(*author);
  vbox3->pack_start(*homepage);
  vbox3->pack_start(*email);
  hbox2->set_border_width(10);
  hbox2->set_spacing(10);
  hbox2->pack_start(*vbox2, Gtk::PACK_SHRINK);
  hbox2->pack_start(*vbox3);
  
  // Place the add button & compiler commands entry in an hbox together
  add->signal_clicked().connect(sigc::mem_fun(*this, &project_properties_dialog::on_add));
  add->set_size_request(53,32);
  hbox->pack_start(*compiler, Gtk::PACK_EXPAND_WIDGET, 5);
  hbox->pack_start(*add, Gtk::PACK_SHRINK, 5);
  
  // Populate the command_list ListStore and load the commands from the Makefile.am
  populate_liststore();
  load_compiler_commands_from_makefile();
  scrolled_win->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  scrolled_win->add(*command_list);
  scrolled_win->set_border_width(5);
  vbox->pack_start(*hbox, Gtk::PACK_SHRINK, 5);
  vbox->pack_start(*scrolled_win, Gtk::PACK_EXPAND_WIDGET);
  
  // Create the notebook and add the tabs to it
  notebook->set_show_tabs(true);
  notebook->set_show_border(true);
  notebook->set_tab_pos(Gtk::POS_TOP);
  notebook->append_page(*hbox2, *t1);
  notebook->append_page(*vbox, *t2);
  
  // Set the dialog properties
  dialog->get_vbox()->set_homogeneous(false);
  dialog->get_vbox()->set_spacing(0);
  dialog->get_vbox()->pack_start(*notebook);
  dialog->set_title("Project Properties");
  dialog->set_default_size(310,350);
  dialog->set_modal(true);
  dialog->property_window_position().set_value(Gtk::WIN_POS_CENTER);
  dialog->set_resizable(true);
  dialog->property_destroy_with_parent().set_value(false);
  dialog->set_has_separator(true);
  dialog->add_action_widget(*cancel, -6);
  dialog->add_action_widget(*ok, -5);
  dialog->set_default(*ok);
  
  // Show the dialog & its widgets
  dialog->show_all_children();
  dialog->show();
}

// Initialize all of the objects in the class
void project_properties_dialog::initialize_objects()
{
  dialog = this;
  cancel = new Gtk::Button(Gtk::StockID("gtk-cancel"));
  ok = new Gtk::Button(Gtk::StockID("gtk-ok"));
  add = new Gtk::Button(Gtk::StockID("gtk-add"));
    
  name = new Gtk::Entry();
  author = new Gtk::Entry();
  email = new Gtk::Entry();
  homepage = new Gtk::Entry();
  version = new Gtk::Entry();
  compiler = new Gtk::Entry();
    
  l1 = new Gtk::Label("Name:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  l2 = new Gtk::Label("Version:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  l3 = new Gtk::Label("Website:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  l4 = new Gtk::Label("E-mail:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  l5 = new Gtk::Label("Author:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER, false);
  t1 = new Gtk::Label("General");
  t2 = new Gtk::Label("Compiler");
   
  hbox = new Gtk::HBox(false, 0);
  vbox = new Gtk::VBox(false, 0);
  hbox2 = new Gtk::HBox(false, 0);
  vbox2 = new Gtk::VBox(false, 0);
  vbox3 = new Gtk::VBox(false, 0);
    
  scrolled_win = new Gtk::ScrolledWindow();
  command_list = new Gtk::TreeView();
  combobox = new Gtk::ComboBoxEntry();
  notebook = new Gtk::Notebook();
}

// Delete widgets when dialog is deleted
project_properties_dialog::~project_properties_dialog()
{
  delete cancel;
  delete ok;
  delete add;
  delete name;
  delete author;
  delete email;
  delete homepage;
  delete version;
  delete compiler;
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete t1;
  delete t2;
  delete hbox;
  delete hbox2;
  delete vbox;
  delete vbox2;
  delete vbox3;
  delete command_list;
  delete scrolled_win;
  delete combobox;
  delete notebook;
}

// Populate the command_list ListStore
void project_properties_dialog::populate_liststore()
{  
  model = Gtk::ListStore::create(columns);
  
  // Add the rows to the ListStore
  add_row("-g", "Produce debugging information in native format");
  add_row("-Idir", "Add a directory to search for header files");
  add_row("-O0", "Do not optimize at all");
  add_row("-O1", "Reduce code size & execution time");
  add_row("-O2", "Optimize without sacrificing speed");
  add_row("-O3", "High level optimization");
  add_row("-Os", "Optimize only to reduce code size");
  add_row("-w", "Suppress all warnings");
  add_row("-Wall", "Turn on all standard warning messages");
  
  // Set the model to the ListStore & append the columns
  command_list->set_model(model);
  command_list->append_column("Command", columns.col_command);
  command_list->append_column("Description", columns.col_description);
}

// Add a row to the ListStore model
void project_properties_dialog::add_row(Glib::ustring comm, Glib::ustring descrip)
{
  Gtk::TreeModel::Row row = *(model->append());
  row[columns.col_command] = comm;
  row[columns.col_description] = descrip;
}

// Load the compiler commands from the Makefile.am
void project_properties_dialog::load_compiler_commands_from_makefile()
{
  Glib::ustring text;
  gint location;
  
  text = read_file(makefile_location);
  
  // Simplify to get out the compiler flags & remove returns and \'s
  location = text.find("FLAGS =", 0);
  text.erase(0, location+7);
  while (text.find("\n", 0) != string::npos)
    text.erase(text.find("\n", 0), 1);
  while (text.find("\\", 0) != string::npos)
    text.erase(text.find("\\", 0), 1);
  location = text.find("bin_PROGRAMS", 0);
  text.erase(location, text.length()-location);
  
  // Simplify white space
  location = text.find("-", 0);
  text.erase(0, location);
  while (text.find("  ", 0) != string::npos)
    text.erase(text.find("  ", 0), 1);
  while (text.find("\t", 0) != string::npos)
    text.erase(text.find("\t", 0), 1);
  
  compiler->set_text(text);
}

// Add the selected compiler command to the compiler entry
void project_properties_dialog::add_compiler_command(Glib::ustring c)
{
  Glib::ustring text = compiler->get_text();
  unsigned int location = text.find(c, 0);
  
  // Check if it already exists
  if (location == string::npos)
  {
    // Add optimization command (Remove others)
    if (c.find("-O", 0) != string::npos)
      if (text.find("-O", 0) != string::npos)
        text.erase(text.find("-O", 0), 4);
      
    // Add the directory to the include command
    else if (c == "-Idir")
      c = "-I/usr/include/directory";
    
    // Make sure warning commands do not conflict
    else if (c == "-Wall" && text.find("-w", 0) != string::npos)
      text.erase(text.find("-w", 0), 3);
    else if (c == "-w" && text.find("-Wall", 0) != string::npos)
      text.erase(text.find("-Wall", 0), 6);
  
    text += " " + c;
    compiler->set_text(text);
  }
}

// Check which command is chosen
void project_properties_dialog::on_add()
{
  Glib::RefPtr<Gtk::TreeSelection> selection = command_list->get_selection();
  Gtk::TreeModel::iterator iter = selection->get_selected();
    
  if (iter)
  {
    Gtk::TreeModel::Row row = *iter;
    Glib::ustring text;
    row->get_value(0, text);
    add_compiler_command(text);
  }
}

// Get projprop properties
Glib::ustring project_properties_dialog::get_name()     { return name->get_text(); }
Glib::ustring project_properties_dialog::get_author()   { return author->get_text(); }
Glib::ustring project_properties_dialog::get_email()    { return email->get_text(); }
Glib::ustring project_properties_dialog::get_homepage() { return homepage->get_text(); }
Glib::ustring project_properties_dialog::get_version()  { return version->get_text(); }
Glib::ustring project_properties_dialog::get_commands() { return compiler->get_text(); }
