/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "settings.h"
#include "utils.h"

#include <iostream>
#include <cstdlib>
#include <gtkmm/main.h>
#include <glib.h>
using namespace std;

bool get_attribute_bool(Glib::ustring, Glib::ustring&, bool);
Glib::ustring get_attribute_ustring(Glib::ustring, Glib::ustring&, Glib::ustring);
int get_attribute_int(Glib::ustring, Glib::ustring&, int);

environment_settings::environment_settings() {}
environment_settings::~environment_settings() {}

// Set syntax color attribute
void environment_settings::set_syntax (int attribute, Glib::ustring color) 
{
  syntax[attribute] = color;
}

// Get user environment settings
void environment_settings::load_settings_file()
{
  Glib::ustring txt, text, config = Glib::get_home_dir() + "/.openldev/openldev.settings";  

  // Open settings file
  text = read_file(config);

  // Get autosave & environment properties
  autosave = get_attribute_int("<autosave>", text, 10);
  width = get_attribute_int("<width>", text, 800);
  height = get_attribute_int("<height>", text, 600);
  treeview_width = get_attribute_int("<tvwidth>", text, 175);
  buildbox_height = get_attribute_int("<bbheight>", text, 175);  
  
  // Get lastproject and boolean properties
  lastproject = get_attribute_ustring("<lastproject>", text, "EMPTY");
  autoload = get_attribute_bool("<autoload>", text, true);
  replace_tabs = get_attribute_bool("<replacetabs>", text, true);
  line_numbers = get_attribute_bool("<lnumbers>", text, true);
  
  // Get compiler color settings
  error = get_attribute_ustring("<error>", text, "#FF0000");
  warning = get_attribute_ustring("<warning>", text, "#0000CC");
  
  // Get general attributes for openldev environment
  tabsize = get_attribute_int("<tabsize>", text, 2);  
  terminal = get_attribute_ustring("<terminal>", text, "xterm");
  fontface = get_attribute_ustring("<fontface>", text, "Monospace");  
  fontsize = get_attribute_ustring("<fontsize>", text, "10");
  toolbar = get_attribute_ustring("<toolbar>", text, "NFOPSVSPCTCOPTSPUDRDSPFDGLSPCLBUEXSTSPPFCPPI");
 
  // Get syntax colors
  syntax[0] = get_attribute_ustring("<variable>", text, "#990000");
  syntax[1] = get_attribute_ustring("<character>", text, "#FF33FF");
  syntax[2] = get_attribute_ustring("<string>", text, "#CC0000");
  syntax[3] = get_attribute_ustring("<comment>", text, "#999999");
  syntax[4] = get_attribute_ustring("<keyword>", text, "#0000CC");
  syntax[5] = get_attribute_ustring("<preprocessor>", text, "#006600");
}

// Write the settings file
void environment_settings::write_settings_file()
{
  Glib::ustring text, fn = Glib::get_home_dir();
  gchar tab[2], ht[5], wt[5], tv[5], bb[5], as[2];
  
  // Convert integer properties to character arrays
  snprintf(tab, sizeof(tab), "%d", tabsize);
  snprintf(ht, sizeof(ht), "%d", height);
  snprintf(wt, sizeof(wt), "%d", width);
  snprintf(tv, sizeof(tv), "%d", treeview_width);
  snprintf(bb, sizeof(bb), "%d", buildbox_height);
  snprintf(as, sizeof(as), "%d", autosave);
    
  // Create .ODP file text
  text = "<autosave>" + Glib::ustring(as) 
       + "*\n<error>" + error 
       + "*\n<warning>" + Glib::ustring(warning) 
       + "*\n<tabsize>" + tab 
       + "*\n<width>" + wt 
       + "*\n<height>" + ht 
       + "*\n<tvwidth>" + tv 
       + "*\n<toolbar>" + toolbar
       + "*\n<bbheight>" + bb 
       + "*\n<autoload>";
  
  if (autoload) text += "T";
  else          text += "F";
    
  if (replace_tabs) text += "*\n<replacetabs>T";
  else              text += "*\n<replacetabs>F";
    
  if (line_numbers) text += "*\n<lnumbers>T";
  else              text += "*\n<lnumbers>F";
    
  text += "*\n<terminal>" + terminal 
       + "*\n<fontface>" + fontface 
       + "*\n<fontsize>" + fontsize 
       + "*\n<lastproject>" + lastproject 
       + "*\n<variable>" + syntax[0] 
       + "*\n<character>" + syntax[1]
       + "*\n<string>" + syntax[2] 
       + "*\n<comment>"  + syntax[3] 
       + "*\n<keyword>" + syntax[4] 
       + "*\n<preprocessor>" + syntax[5] + "*\n";
  
  fn += "/.openldev/openldev.settings";
  
  // Write the file
  write_file(fn, text);
}

void environment_settings::create_new_settings_file()
{
  set_autosave(10);
  set_error("#FF0000");
  set_warning("#0000CC");
  set_tabsize(2);
  set_autoload(true);
  set_replace_tabs(true);
  set_line_numbers(true);
  set_terminal("xterm");
  set_font_face("Monospace");
  set_font_size("10");
  set_toolbar("NFOPSVSPCTCOPTSPUDRDSPFDGLSPCLBUEXSTSPPFCPPI");
  set_lastproject("EMPTY");
  set_syntax(0, "#990000");
  set_syntax(1, "#FF33FF");
  set_syntax(2, "#CC0000");
  set_syntax(3, "#999999");
  set_syntax(4, "#0000CC");
  set_syntax(5, "#006600");
  set_width(800);
  set_height(600);
  set_treeview_width(175);
  set_buildbox_height(175);
  write_settings_file();
}

// Get boolean data from the specified attribute in text
bool get_attribute_bool(Glib::ustring attribute, Glib::ustring &text, bool fail)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    Glib::ustring txt = text.substr(start, len);
    if (txt == "T")
      return true;
    else
      return false;
  }
  else
    return fail;
}

// Get ustring data from the specified attribute in text
Glib::ustring get_attribute_ustring(Glib::ustring attribute, Glib::ustring &text, Glib::ustring fail)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    return text.substr(start, len);
  }
  else
    return fail;
}

// Get integer data from the specified attribute in text
int get_attribute_int(Glib::ustring attribute, Glib::ustring &text, int fail)
{
  int start, len;
  
  if (text.find(attribute, 0) != string::npos)
  {
    start = text.find(attribute, 0) + attribute.length();
    len = text.find("*", start) - start;
    Glib::ustring txt = text.substr(start, len);
    locale_from_utf8(txt);
    return atol(txt.c_str());
  }
  else
    return fail;
}
  
// Set variables
void environment_settings::set_autoload   (bool load)           { autoload = load; }
void environment_settings::set_replace_tabs (bool var)          { replace_tabs = var; }
void environment_settings::set_line_numbers (bool var)          { line_numbers = var; }
void environment_settings::set_autosave   (int save)            { autosave = save; }
void environment_settings::set_tabsize    (int size)            { tabsize = size; }
void environment_settings::set_width      (int size)            { width = size; }
void environment_settings::set_height     (int size)            { height = size; }
void environment_settings::set_treeview_width  (int size)       { treeview_width = size; }
void environment_settings::set_buildbox_height (int size)       { buildbox_height = size; }
void environment_settings::set_font_size  (Glib::ustring size)  { fontsize = size; }
void environment_settings::set_error      (Glib::ustring color) { error = color; }
void environment_settings::set_warning    (Glib::ustring color) { warning = color; }
void environment_settings::set_terminal   (Glib::ustring term)  { terminal = term; }
void environment_settings::set_font_face  (Glib::ustring face)  { fontface = face; }
void environment_settings::set_lastproject(Glib::ustring name)  { lastproject = name; }
void environment_settings::set_toolbar    (Glib::ustring tb)    { toolbar = tb; }

// Get syntax color of specified attribute
Glib::ustring environment_settings::get_syntax_color (int attribute) 
{
  return syntax[attribute];
}  

// Get variable values
int environment_settings::get_autosave()        { return autosave; }
int environment_settings::get_tabsize()         { return tabsize; }
int environment_settings::get_width()           { return width; }
int environment_settings::get_height()          { return height; }
int environment_settings::get_treeview_width()  { return treeview_width; }
int environment_settings::get_buildbox_height() { return buildbox_height; }
bool environment_settings::get_autoload()       { return autoload; }
bool environment_settings::get_replace_tabs()   { return replace_tabs; }
bool environment_settings::get_line_numbers()   { return line_numbers; }
 
Glib::ustring environment_settings::get_font_size()     { return fontsize; }
Glib::ustring environment_settings::get_error_color()   { return error; }
Glib::ustring environment_settings::get_warning_color() { return warning; }
Glib::ustring environment_settings::get_terminal()      { return terminal; }
Glib::ustring environment_settings::get_font_face()     { return fontface; }
Glib::ustring environment_settings::get_lastproject()   { return lastproject; }
Glib::ustring environment_settings::get_toolbar()       { return toolbar; }
