/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "syntax.h"
#include "utils.h"

Glib::ustring varKey[14] = {"bool", "char", "const", "double", "float", "int", "long", "short", "signed", "static", "unsigned", "void", "virtual", "wchar_t"};
Glib::ustring wrdKey[50] = {"asm", "auto", "break", "case", "catch", "class", "const_cast", "continue", "default", "delete", "do", "dynamic_cast", "else", "enum", "explicit", "extern", "false", "for", "friend", "goto", "if", "inline", "mutable", "namespace", "new", "null", "operator", "private:", "protected:", "public:", "register", "reinterpret_cast", "return", "sizeof", "static_cast", "struct", "switch", "template", "this", "throw", "true", "try", "typedef", "typeid", "typename", "union", "using", "virtual", "volatile", "while"}; 
  
bool is_acceptable_char (char ch)
{
  if (ch == '&' || ch == '/' || ch == ' ' || ch == '(' || ch == '\t' ||
      ch == ')' || ch == ';' || ch == '*' || ch == '\n' || ch == ':')
    return true;

  return false;
}

bool is_keyword_or_variable (Gtk::TextIter previous, Gtk::TextIter next)
{
  bool previous_test = false, next_test = false;
  
  if (previous.is_start())
    previous_test = true;
  else
  {
    previous--;
    if (is_acceptable_char(previous.get_char()))
      previous_test = true;
  }
    
  if (next.is_end())
    next_test = true;
  else
  {
    if (is_acceptable_char(next.get_char()))
      next_test = true;
  }
  
  if (previous_test && next_test)
    return true;
  
  return false;
}
  
// Hightlight c/c++ keywords and variable keywords
void set_syntax_keywords(Glib::RefPtr<Gtk::TextBuffer> text_buffer,
                         Glib::RefPtr<Gtk::TextBuffer::Tag> vtyp,
                         Glib::RefPtr<Gtk::TextBuffer::Tag> kywd,
                         Gtk::TextIter begin, Gtk::TextIter enditer)
{
  Gtk::TextIter end, iter = begin, previous, next;

  // Find & mark all keywords
  for (int i=0; i<50; i++) 
  {
    while (iter.forward_search(wrdKey[i],Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
    {
      if (is_keyword_or_variable(iter, end))
        text_buffer->apply_tag(kywd,iter,end);
      
      iter++;
    }
    iter = begin;
  }
  
  // Find & mark all variable types
  for (int j=0; j<14; j++) 
  {
    while (iter.forward_search(varKey[j],Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
    {
      if (is_keyword_or_variable(iter, end))
        text_buffer->apply_tag(vtyp,iter,end);

      iter++;
    }
    iter = begin;
  }
}

// Mark all strings and characters
void set_syntax_string_char(Glib::RefPtr<Gtk::TextBuffer> text_buffer,
                            Glib::RefPtr<Gtk::TextBuffer::Tag> strn,
                            Glib::RefPtr<Gtk::TextBuffer::Tag> chr,
                            Gtk::TextIter begin, Gtk::TextIter enditer)
{
  Gtk::TextIter end, iter = begin, start;
  
  // Mark all strings
  while (iter.forward_search("\"",Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
  {
    start = iter;
    start++;
    if (start.forward_search("\"",Gtk::TEXT_SEARCH_VISIBLE_ONLY,start,end,enditer)) 
    {
      // Only apply if the string spans only one line
      if (end.get_line() == iter.get_line()) 
      {
        text_buffer->apply_tag(strn,iter,end);
        iter = end;
      } 
      else 
        iter++;
    } 
    else 
      break;
  }
  iter = begin;
  
  // Mark all characters (those between 's)
  while (iter.forward_search("'",Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
  {
    start = iter;
    start++;
    if (start.forward_search("'",Gtk::TEXT_SEARCH_VISIBLE_ONLY,start,end,enditer)) 
    {
      // Only apply if character spans only one line
      if (end.get_line() == iter.get_line()) 
      {
        text_buffer->apply_tag(chr,iter,end);
        iter = end;
      } 
      else 
        iter++;
    } 
    else
      break;
  }
}

// Mark preprocessor directives
void set_syntax_preprocessor(Glib::RefPtr<Gtk::TextBuffer> text_buffer,
                            Glib::RefPtr<Gtk::TextBuffer::Tag> prep,
                            Gtk::TextIter begin, Gtk::TextIter enditer)
{
  Gtk::TextIter end, iter = begin;
  bool test = false;
  
  // Mark all preprocessor directives
  while (iter.forward_search("#",Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
  {
    test = true;
    for (int i=iter.get_offset()-1; i>=iter.get_offset()-iter.get_line_offset(); i--) 
    {
      text_buffer->get_iter_at_offset(i);
      // Check to make sure the '#' is the 1st non-alphanumeric character
      if (*iter != '\0' && *iter != ' ') 
      {
        test = false;
        break;
      }
    }
    // Apply if the conditions are met
    if (test) 
    {
      end.forward_to_line_end();
      text_buffer->apply_tag(prep,iter,end);
    }
    iter++;
  }
}

// Mark single line comments
void set_syntax_single_cmmnt(Glib::RefPtr<Gtk::TextBuffer> text_buffer,
                            Glib::RefPtr<Gtk::TextBuffer::Tag> cmnt,
                            Gtk::TextIter begin, Gtk::TextIter enditer)
{
  Gtk::TextIter end, iter = begin, temp;
  bool test = false;
  
  // Mark one-line comments
  while (iter.forward_search("//",Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
  {
    int before=0, after=0, i;
    test = true;
    
    // Search, skipping comments in a string
    for (i=iter.get_offset(); i>=iter.get_offset()-iter.get_line_offset(); i--) 
    {
      begin = text_buffer->get_iter_at_offset(i);
      if (*begin == '\"')
        before++;
    }
    temp = end;
    
    // Find the end of the line
    temp.forward_search("\n",Gtk::TEXT_SEARCH_VISIBLE_ONLY,temp,begin,enditer);
    for (i=iter.get_offset(); i<=temp.get_offset(); i++) 
    {
      begin = text_buffer->get_iter_at_offset(i);
      if (*begin == '\"')
        after++;
    }
    
    // Make sure it is not in a string
    if (before%2==1 && after%2==1)
      test = false;
      
    // If tests are satisfied, apply highlighting
    if (test)
    {
      end.set_line(iter.get_line()+1);
      text_buffer->apply_tag(cmnt,iter,end);
    }
    iter++;
  }
}

// Mark double line comments
void set_syntax_double_cmmnt(Glib::RefPtr<Gtk::TextBuffer> text_buffer,
                            Glib::RefPtr<Gtk::TextBuffer::Tag> cmnt,
                            Gtk::TextIter begin, Gtk::TextIter enditer)
{
  Gtk::TextIter end, iter = begin, temp;
  bool test = false;
  
  // Mark multi-line comments
  while (iter.forward_search("/*",Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
  {
    int before=0, after=0, i;
    test = true;
    
    // Make sure it is not in a string
    for (i=iter.get_offset(); i>=iter.get_offset()-iter.get_line_offset(); i--) 
    {
      begin = text_buffer->get_iter_at_offset(i);
      if (*begin == '\"')
        before++;
    }
    temp = end;
    
    // Make sure the end is not in a string
    temp.forward_search("\n",Gtk::TEXT_SEARCH_VISIBLE_ONLY,temp,begin,enditer);
    for (i=iter.get_offset(); i<=temp.get_offset(); i++) 
    {
      begin = text_buffer->get_iter_at_offset(i);
      if (*begin == '\"')
        after++;
    }
    
    // Check if tests are satisfied
    if (before%2==1 && after%2==1)
      test = false;
      
    // Apply if the comment is not in a string
    if (test) 
    {
      if (iter.forward_search("*/",Gtk::TEXT_SEARCH_VISIBLE_ONLY,begin,end,enditer))
        text_buffer->apply_tag(cmnt,iter,end);
      else
        text_buffer->apply_tag(cmnt,iter,enditer);
    }
    iter = begin;  
  }
}

// Calculate syntax highlighting & update text_buffer
void set_syntax(Glib::RefPtr<Gtk::TextBuffer> text_buffer, bool apply,
                bool redo_all_highlighting, environment_settings &env)
{
  Gtk::TextIter iter = text_buffer->begin();
  Gtk::TextIter enditer = text_buffer->end();
  Gtk::TextIter begin = iter, end = enditer;
  
  // Refresh the text buffer & create the tag table
  Glib::RefPtr<Gtk::TextBuffer::Tag> vtyp, chr, strn, cmnt, kywd, prep;
  Glib::RefPtr<Gtk::TextBuffer::TagTable> tag_table = text_buffer->get_tag_table();  

  // If tags must be created
  if (apply) 
  {  
    // Create color tags for syntax highlighting
    vtyp = Gtk::TextBuffer::Tag::create("Variables");
    vtyp->property_foreground().set_value(env.get_syntax_color(0));
    chr  = Gtk::TextBuffer::Tag::create("Characters");
    chr->property_foreground().set_value(env.get_syntax_color(1));
    strn = Gtk::TextBuffer::Tag::create("Strings");
    strn->property_foreground().set_value(env.get_syntax_color(2));
    cmnt = Gtk::TextBuffer::Tag::create("Comments");
    cmnt->property_foreground().set_value(env.get_syntax_color(3));
    kywd = Gtk::TextBuffer::Tag::create("Keywords");
    kywd->property_foreground().set_value(env.get_syntax_color(4));
    prep = Gtk::TextBuffer::Tag::create("Preprocessor");
    prep->property_foreground().set_value(env.get_syntax_color(5)); 
        
    // Add tags to tag-table
    tag_table->add(vtyp);
    tag_table->add(kywd);
    tag_table->add(chr);
    tag_table->add(strn);
    tag_table->add(prep);
    tag_table->add(cmnt);
  }
  // If tags are already created, just reference them
  else 
  {
    vtyp = tag_table->lookup("Variables");
    chr  = tag_table->lookup("Characters");
    strn = tag_table->lookup("Strings");
    cmnt = tag_table->lookup("Comments");
    kywd = tag_table->lookup("Keywords");
    prep = tag_table->lookup("Preprocessor");
  }
  
  // If the document is being opened for the first time, 
  //  apply tags to whole document
  if (redo_all_highlighting)
  { 
    text_buffer->remove_all_tags(text_buffer->begin(), text_buffer->end());
    set_syntax_keywords(text_buffer, vtyp, kywd, iter, enditer);
    set_syntax_string_char(text_buffer, strn, chr, iter, enditer);
    set_syntax_preprocessor(text_buffer, prep, iter, enditer);  
    set_syntax_single_cmmnt(text_buffer, cmnt, iter, enditer);
  }
  // Update tags in the document
  else
  {
    // Update the keywords
    begin = text_buffer->get_insert()->get_iter();
    end = begin;
    begin.backward_word_start();
    begin--;
    end.forward_word_end();
    end++;
    text_buffer->remove_tag(vtyp, begin, end);
    text_buffer->remove_tag(kywd, begin, end);
    set_syntax_keywords(text_buffer, vtyp, kywd, begin, end);
    
    // Remove other tags from the document
    begin = text_buffer->get_insert()->get_iter();
    end = begin;
    begin.backward_line();
    end.forward_line();
    end.forward_to_line_end();
    text_buffer->remove_tag(chr, begin, end);
    text_buffer->remove_tag(strn, begin, end);
    text_buffer->remove_tag(cmnt, begin, end);
    text_buffer->remove_tag(prep, begin, end);
    
    // Update other syntax highlighting
    set_syntax_string_char(text_buffer, strn, chr, begin, end);
    set_syntax_preprocessor(text_buffer, prep, begin, end);  
    set_syntax_single_cmmnt(text_buffer, cmnt, begin, end);
  }
  
  set_syntax_double_cmmnt(text_buffer, cmnt, iter, enditer);  
}
